#!/bin/bash
#
# TestSuite for xine-lib
#

export TESTSUITE_VERSION=0.2.4-valtri
export TESTSUITE_DIR=$(cd $(dirname $0)/..; pwd)
export TEST_DIR=${TESTSUITE_DIR}/tests
export RESULTS_DIR=${TESTSUITE_DIR}/results
export REPORTS_DIR=${TESTSUITE_DIR}/reports
export TEMPLATE_DIR=${TESTSUITE_DIR}/templates
export PLATFORM=${1:-native}
[ -f ${TESTSUITE_DIR}/bin/config.sh ] && . ${TESTSUITE_DIR}/bin/config.sh
. ${TESTSUITE_DIR}/bin/functions.sh
export PLATFORM_NAME=${PLATFORM_NAME:-"$(uname) HEAD"}
export PLATFORM_ID=${PLATFORM_ID:-"$(uname | tr \"[:upper:]\" \"[:lower:]\")"}
export TEST_LIST=${TEST_LIST:-${TEST_DIR}/test-${PLATFORM}.list}
export RESULT_DIR=${RESULT_DIR:-${RESULTS_DIR}/${PLATFORM_ID}}
STARTDATE=$(${date})
ISODATE=$(${date} --iso-8601)

echo

RUN_REPORT_DIR=${REPORTS_DIR}/${PLATFORM_ID}
REPORT_DIR=${REPORTS_DIR}/${PLATFORM_ID}/${ISODATE}


# create the report directory
check_dir "${REPORT_DIR}"

# cleanup the report directory
rm -rf "${REPORT_DIR}"/*

# create trace file
init_trace

trace "TestSuite for xine-lib"
trace "Version ${TESTSUITE_VERSION}"
trace 
trace "Configuration:"
trace "  SAMPLE_DIR: ${SAMPLE_DIR}"
trace "  REPORT_DIR: ${REPORT_DIR}"
trace

toxine=$(eval echo "$toxine_cmd")
export toxine

# check deps
if [ "${systoxine}" == "" ]; then
  trace "system toxine must be specified"
  exit 1
fi
if [ "${time}" == "" ]; then
  trace "time must be installed"
  exit 1
fi
if [ "${find}" == "" ]; then
  trace "find must be installed"
  exit 1
fi
if [ "${sort}" == "" ]; then
  trace "sort must be installed"
  exit 1
fi
if [ "${bc}" == "" ]; then
  trace "bc must be installed"
  exit 1
fi
if ! which dos2unix 2>&1 >/dev/null; then
  trace "dos2unix required"
  exit 1
fi

if test ! -d "${SAMPLE_DIR}"; then
  trace "Sample directory '${SAMPLE_DIR}' doesn't exists"
  exit 1
fi

function executeTests {
    # cleanup the result dir
    check_dir "${RESULT_DIR}"
    rm -rf "${RESULT_DIR}"/*
    
    # launch all tests
    trace "Test running..."
    echo "${PLATFORM_NAME}" > "${RESULT_DIR}/.platform"
    echo $ISODATE > "${RESULT_DIR}/.date"
    for testBasename in $(< ${TEST_LIST});
      do
      testResultDir=${RESULT_DIR}/${testBasename}
      playlistTypeFile=${testResultDir}/playlistType 
      
      trace
      trace "Launching test: ${testBasename}"
      
      # create the resultDir for this test
      mkdir -p "${testResultDir}"
      if [ $? -ne 0 ]; then
	  trace "Can't create dir ${testResultDir}"
	  exit 1
      fi

      # get the playlist
      playlistFile=${TEST_DIR}/$(dirname $testBasename)/$(basename $testBasename .test).playlist
  
      unset playlistMRLs
      unset playlistTitles
      declare -a playlistMRLs
      declare -a playlistTitles
      if [ -f "$playlistFile" ]; then
	  getPlaylistFile
	  echo "pls" > ${playlistTypeFile}
      else
	  getDefaultPlaylist
	  echo "default" > ${playlistTypeFile}
      fi
      
      i=0
      while [ ${i} -lt ${#playlistMRLs[*]} ];
	do
	sampleId=""
	if [ ${i} -lt 10 ]; then
	    sampleId="00"
	elif [ ${i} -lt 100 ]; then
	    sampleId="0"
	fi
	sampleId="sample_${sampleId}${i}"
	sampleMRL="${playlistMRLs[${i}]}"
	sampleTitle="${playlistTitles[${i}]}"
	sampleResultDir="${testResultDir}/${sampleId}"
	resultFile="${sampleResultDir}/result"
	stdoutFile="${sampleResultDir}/stdout"
	stderrFile="${sampleResultDir}/stderr"
	timeFile="${sampleResultDir}/duration"
	titleFile="${sampleResultDir}/title"
	
        # create the resultDir for this sample
	mkdir -p "${sampleResultDir}"
	if [ $? -ne 0 ]; then
	    trace "Can't create dir ${sampleResultDir}"
	    exit 1
	fi
  
        # read sample config file
	unset test_args
	declare -a test_args
	loadSampleConfigFile "${testBasename}" "${sampleMRL}"
  
	trace "  Testing sample: ${sampleMRL}"
	LC_ALL=C "${time}" -p bash -c "\"${TEST_DIR}/${testBasename}\" \"${sampleMRL}\" ${test_args[@]} 1>\"${stdoutFile}\" 2>\"${stderrFile}\"" 2>"${timeFile}"
#	LC_ALL=C "${time}" -p -o "${timeFile}" bash -x -c "\"${TEST_DIR}/${testBasename}\" \"${sampleMRL}\" ${test_args[@]} 1>\"${stdoutFile}\" 2>\"${stderrFile}\"" 2>"${sampleResultDir}/bashlog"
	result=$?

	echo "define p(a,s) { scale=s; foo=(a+.5/10^s)/1; if (foo<=9) print 0; return foo/1}; a=$(grep real ${timeFile} | cut -f 2 -d ' '); print p(a/60,0); \":\"; print p(a%60,2)" | ${bc} > "${timeFile}".2
	mv "${timeFile}".2 "${timeFile}"

	grep 'wine: Unhandled exception .*, starting debugger...' "${stderrFile}"
	[ "$?" -eq 0 ] && result="3"
	echo "${result}" > "${resultFile}"
 
	if [ "${result}" -eq 0 ]; then
	    trace "  Result: OK"
	else
	    trace "  Result: KO"
	fi
  
	echo "${sampleTitle}" > "${titleFile}"
	
	i=$[${i} + 1]
      done
      trace "Test finished"
    done
    ENDDATE=$(${date})
}

function generateReport {
    # ---------------------
    # Generate the report
    # ---------------------

    TEST_COUNT=0
    TEST_OK=0
    TEST_KO=0
    DEADLOCKS=0
    SEGFAULTS=0

    trace "Building the report"
    RUN_RESULT_FILE="${REPORT_DIR}/summary.txt"
    RUN_NEW_FAILURE_FILE="${REPORT_DIR}/new_failure.txt"
    RUN_SUCESS_RATE_FILE="${REPORT_DIR}/success_rate.txt"
    PREVIOUS_RUN=$(getPreviousRun)
    touch "${RUN_RESULT_FILE}"
    touch "${RUN_NEW_FAILURE_FILE}"
    touch "${RUN_SUCESS_RATE_FILE}"
    # FIXME: some bug on run 14. 9. 2006 (returned dir before 13. 9.)
    trace "Previous run: $PREVIOUS_RUN"

    for testBasename in $(< "${TEST_LIST}");
      do
      testResultDir="${RESULT_DIR}/${testBasename}"
      testReportDir="${REPORT_DIR}/${testBasename}"
      testReportFile="${testReportDir}/${testBasename}.html"
      playlistTypeFile="${testResultDir}/playlistType"

      trace "Test: ${testBasename}"

      # create a directory for this test
      mkdir -p "${testReportDir}"

      # copy script source
      cp "${TEST_DIR}/${testBasename}" "${testReportDir}/"
      if [ $? -ne 0 ]; then
	  echo "Failed to copy ${testBasename}"
	  exit 1
      fi

      # define variables for templates
      TEST_NAME="${testBasename}"
  
      # generates test header
      PLAYLIST_TYPE=$(< "${playlistTypeFile}")
      if [ "${PLAYLIST_TYPE}" == "default"  ]; then
	  process_template test_header_default.tmpl "${testReportFile}"
      else
	  process_template test_header.tmpl "${testReportFile}"
      fi
  
      SAMPLES_COUNT=0
      SAMPLES_OK=0
      SAMPLES_KO=0

      ls ${testResultDir}/sample* >/dev/null 2>/dev/null && \
      for sampleResultDir in ${testResultDir}/sample*;
	do
	sampleId=$(basename ${sampleResultDir})
	sampleTitle=$(< "${sampleResultDir}/title")
	sampleReportDir=${testReportDir}/${sampleId}
	resultFile=${sampleResultDir}/result
	stdoutFile=${sampleResultDir}/stdout
	stderrFile=${sampleResultDir}/stderr
	timeFile=${sampleResultDir}/duration
	
        # create the directory for this sample
	mkdir -p "${sampleReportDir}"
	if [ $? -ne 0 ]; then
	    trace "Failed to create ${sampleReportDir} dir"
	    exit 1
	fi
  
        # copy stdout and stderr
	cp "${stdoutFile}" "${sampleReportDir}/"
	if [ $? -ne 0 ]; then
	    trace "Failed to copy ${stdoutFile}"
	    exit 1
	fi
	cp "${stderrFile}" "${sampleReportDir}/"
	if [ $? -ne 0 ]; then
	    trace "Failed to copy ${stderrFile}"
	    exit 1
	fi
	
	SAMPLES_COUNT=$[${SAMPLES_COUNT}+1]
  
        # template variables
	SAMPLE_NAME=$(insertSpaces "${sampleTitle}")
	SAMPLE_URL=$(echo "${SAMPLE_BASEURL}/${sampleTitle}" | sed -e 's/ /%20/g')
	SAMPLE_STATUS=$(< "${resultFile}")
	SAMPLE_MESSAGE=$(${systoxine} --error-codes="${SAMPLE_STATUS}")
	SAMPLE_STDOUT=${sampleId}/stdout
	SAMPLE_STDOUT_TEXT="StdOut"
	if [ ! -s "${stdoutFile}" ]; then
	    SAMPLE_STDOUT_TEXT=""
	fi
	SAMPLE_STDERR=${sampleId}/stderr
	SAMPLE_STDERR_TEXT="StdErr"
	if [ ! -s "${stderrFile}" ]; then
	    SAMPLE_STDERR_TEXT=""
	fi
	SAMPLE_TIME=$(< "${timeFile}")

        # sample infos if available
	if [ "${PLAYLIST_TYPE}" = "default"  ]; then
	    TEST_TMPL_PREFIX="test_step_default"
	    SAMPLE_CONTAINER=$(getSampleInfo "${sampleTitle}" "container")
	    SAMPLE_VIDEO_CODEC=$(getSampleInfo "${sampleTitle}" "video_codec")
	    SAMPLE_AUDIO_CODEC=$(getSampleInfo "${sampleTitle}" "audio_codec")
	    SAMPLE_SIZE=$(getSampleInfo "${sampleTitle}" "size")
	else
	    TEST_TMPL_PREFIX="test_step"
	    SAMPLE_CONTAINER="&nbsp;"
	    SAMPLE_VIDEO_CODEC="&nbsp;"
	    SAMPLE_AUDIO_CODEC="&nbsp;"
	    SAMPLE_SIZE="&nbsp;"
	fi
 
	if [ "${SAMPLE_STATUS}" -eq 0 ]; then
	    process_template "${TEST_TMPL_PREFIX}_ok.tmpl" "${testReportFile}"
	    SAMPLES_OK=$[${SAMPLES_OK}+1]
	else
	    process_template "${TEST_TMPL_PREFIX}_ko.tmpl" "${testReportFile}"
	    SAMPLES_KO=$[${SAMPLES_KO}+1]
	    if [ "${SAMPLE_MESSAGE}" = "TOX_ERR_DEADLOCK" ]; then
		DEADLOCKS=$[${DEADLOCKS}+1]
	    elif [ "${SAMPLE_MESSAGE}" = "TOX_ERR_SIGSEGV" ]; then
		SEGFAULTS=$[${SEGFAULTS}+1]
	    fi
	fi
      done
  
      TEST_SUCCESS_PCT=$(echo "scale=2; (100 * ${SAMPLES_OK} / ${SAMPLES_COUNT} + 0.005) / 1" | ${bc})
      TEST_SUCCESS_PCT_INT=$( echo "scale=0; print (${TEST_SUCCESS_PCT} + 0.5) / 1" | ${bc})
      TEST_COUNT=$[${TEST_COUNT}+1]
      
      if [ "${PLAYLIST_TYPE}" = "default"  ]; then
	  process_template test_footer_default.tmpl "${testReportFile}"
      else
	  process_template test_footer.tmpl "${testReportFile}"
      fi

      PREVIOUS_RUN_RESULT=$(getTestResult "${PREVIOUS_RUN}" "${TEST_NAME}")
      PREVIOUS_RUN_RATE_SUCESS=$(getTestRateSuccess "${PREVIOUS_RUN}" "${TEST_NAME}")
      
      echo "${TEST_NAME}: ${TEST_SUCCESS_PCT}" >> "${RUN_SUCESS_RATE_FILE}"
      if [ ${SAMPLES_KO} -eq 0 ]; then
          # the test is a success
	  echo "${TEST_NAME}: success" >> "${RUN_RESULT_FILE}"
          # check if it was a success in the previous run
	  if [ ! "${PREVIOUS_RUN_RESULT}" = "failure" ]; then
          # was a success or empty
	      process_template main_test_ok_equal.tmpl "${REPORT_DIR}/index.html.tmp"
	  else
	      process_template main_test_ok_greater.tmpl "${REPORT_DIR}/index.html.tmp"
	  fi
	  TEST_OK=$[${TEST_OK}+1]
      else
          # the test is a failure
	  echo "${TEST_NAME}: failure" >> "${RUN_RESULT_FILE}"
          # check if it was a success in the previous run
	  if [ "${PREVIOUS_RUN_RESULT}" = "success" ]; then
              # was a success
	      echo "${TEST_NAME}" >> "${RUN_NEW_FAILURE_FILE}"
	  fi

	  if [ "${PREVIOUS_RUN_RATE_SUCESS}" = "" ]; then
	      PREVIOUS_RUN_RATE_SUCESS="${TEST_SUCCESS_PCT}"
	  fi
	  if [ $(echo "${PREVIOUS_RUN_RATE_SUCESS} < ${TEST_SUCCESS_PCT}" | ${bc}) -eq 1 ]; then
	      process_template main_test_ko_greater.tmpl "${REPORT_DIR}/index.html.tmp"
	  elif [ $(echo "${PREVIOUS_RUN_RATE_SUCESS} > ${TEST_SUCCESS_PCT}" | ${bc}) -eq 1 ]; then
	      process_template main_test_ko_lower.tmpl "${REPORT_DIR}/index.html.tmp"
	  else
	      process_template main_test_ko_equal.tmpl "${REPORT_DIR}/index.html.tmp"
	  fi
	  TEST_KO=$[${TEST_KO}+1]
      fi
    done

    # generate summary page
    trace "Generate summary page"
    NEW_FAILED_TESTS=$(wc -l <"${RUN_NEW_FAILURE_FILE}")
    rm -f "${REPORT_DIR}/index.html"
    touch "${REPORT_DIR}/index.html"
    process_template main_header.tmpl "${REPORT_DIR}/index.html"
    cat "${REPORT_DIR}/index.html.tmp" >> "${REPORT_DIR}/index.html"
    rm -f "${REPORT_DIR}/index.html.tmp"
    process_template main_footer.tmpl "${REPORT_DIR}/index.html"
}


function removeOldRuns {
    # remove old runs
    trace "Remove old Runs"
    NOW=$(${date} "+%s")
    for runDir in `ls -1r ${RUN_REPORT_DIR}`;
      do
      if [ -d "${RUN_REPORT_DIR}/${runDir}" ]; then
	  CURRENT_RUN_DATE=$(${date} -d "${runDir}" '+%s')
	  DATE_DIFF=$(( ${NOW} - ${CURRENT_RUN_DATE} ))
	  DAY_DIFF=$(( ${DATE_DIFF} / 86400 ))
	  
	  if [ ${DAY_DIFF} -gt 30 ]; then
	      if [ ${DAY_DIFF} -lt 90 ]; then
		  DAY_OF_WEEK=$(${date} -d "$RUN_DIR" '+%u')
		  if [ ${DAY_OF_WEEK} -ne 1 ]; then
		      rm -rf "${RUN_REPORT_DIR}/${runDir}"
		  fi
	      else
		  DAY_OF_MONTH=$(${date} -d "$RUN_DIR" '+%e')
		  if [ ${DAY_OF_MONTH} -gt 7 ]; then
		      rm -rf "${RUN_REPORT_DIR}/${runDir}"
		  fi
	      fi
	  fi
      fi
    done
}

function generateIndex {
    # regenerate run index page
    trace "Generate index page"
    echo "" > "${RUN_REPORT_DIR}/index.html"
    process_template index_header.tmpl "${RUN_REPORT_DIR}/index.html"
    for runDir in `ls -1r ${RUN_REPORT_DIR}`;
      do
      if [ -d ${RUN_REPORT_DIR}/${runDir} ]; then
	  RUN_DIR=${runDir}
	  RUN_DATE=$(${date} -d ${runDir} '+%A %B %e  %Y')
	  process_template index_run.tmpl "${RUN_REPORT_DIR}/index.html"
      fi
    done

    process_template index_footer.tmpl "${RUN_REPORT_DIR}/index.html"
}

function cleanupResultDir {
    trace "Cleaning result dir..."
    rm -rf "${RESULT_DIR}/*"
}


executeTests
generateReport
cleanupResultDir
removeOldRuns
generateIndex
